/*
 * Copyright (c) 1996, 2003 VIA Networking Technologies, Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *    This product includes software developed by VIA Networking Technologies, Inc.
 * 4. Neither the name of the Company nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY VIA NETWORKING TECHNOLOGIES, INC. AND 
 * CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, 
 * BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL 
 * VIA NETWORKING TECHNOLOGIES, INC. OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *
 * File: if_getb.h
 *
 * Purpose: FreeBSD driver main header file
 *
 * Author: Ryan Fu
 *
 * Date: Jul 22, 2003
 *
 *
 */

#ifndef __IF_GETB_H__
#define __IF_GETB_H__

#include <sys/param.h>

#if __FreeBSD_version < 400000
#include "bpfilter.h"
#endif /*__FreeBSD_version*/

#include <sys/systm.h>
#include <sys/sockio.h>
#include <sys/mbuf.h>
#include <sys/malloc.h>
#if __FreeBSD_version >= 400000
#include <sys/module.h>
#endif
#include <sys/kernel.h>
#include <sys/socket.h>
#include <sys/protosw.h>
#include <sys/types.h>


#include <net/if.h>
#include <net/if_arp.h>
#include <net/ethernet.h>
#include <net/if_dl.h>
#include <net/if_media.h>

#if __FreeBSD_version >= 500000
#include <net/if_vlan_var.h>
#endif

#if (__FreeBSD_version < 400000 && NBPFILTER > 0) || __FreeBSD_version >= 400000
#include <net/bpf.h>
#endif /*__FreeBSD_version*/

#if __FreeBSD_version >=400000 && __FreeBSD_version <410000
#include "opt_bdg.h"
#ifdef BRIDGE
#include <net/bridge.h>
#endif /* BRIDGE */
#endif /*__FreeBSD_version*/

#include <vm/vm.h>
#include <vm/pmap.h>
#include <machine/clock.h>
#include <machine/bus_pio.h>
#include <machine/bus_memio.h>
#include <machine/bus.h>
#include <machine/atomic.h>

#if __FreeBSD_version >= 400000
#include <machine/resource.h>
#include <sys/bus.h>
#include <sys/rman.h>
#endif /*__FreeBSD_version*/

#if __FreeBSD_version >= 502000
#include <dev/pci/pcireg.h>
#include <dev/pci/pcivar.h>
#else
#include <pci/pcireg.h>
#include <pci/pcivar.h>
#endif

#include <netinet/in_systm.h>
#include <netinet/in.h>
#include <netinet/ip.h>
#include <netinet/tcp.h>
#include <netinet/udp.h>

#include "velocity_hw.h"
#include "if_getb_cfg.h"
#include "velocity_desc.h"
#include "velocity_mac.h"
#include "velocity_mii.h"
#include "if_getb_dbg.h"


#define GETB_TX_TIMEOUT                   5

#define GETB_ADD_ONE_WRAP(x, mod)         ( (x) = ((x) + 1) % (mod) )

#define GETB_SUB_ONE_WRAP(x, mod) {       \
    if ((x) <= 0)                         \
        (x) = ((mod) - 1);                \
    else                                  \
        (x)--;                            \
}


#define GETB_WOL_MAGIC              0x00000000UL
#define GETB_WOL_PHY                0x00000001UL
#define GETB_WOL_ARP                0x00000002UL
#define GETB_WOL_UCAST              0x00000004UL
#define GETB_WOL_BCAST              0x00000010UL
#define GETB_WOL_MCAST              0x00000020UL
#define GETB_WOL_MAGIC_SEC          0x00000040UL

/*flags for options*/
#define GETB_FLAGS_VLANTAG          0x00000001UL
#define GETB_FLAGS_TX_CSUM          0x00000002UL
#define GETB_FLAGS_RX_CSUM          0x00000004UL
#define GETB_FLAGS_VAL_PKT_LEN      0x00000008UL

/*flags for driver status*/
#define GETB_FLAGS_OPENED           0x00010000UL
#define GETB_FLAGS_VMNS_CONNECTED   0x00020000UL
#define GETB_FLAGS_VMNS_COMMITTED   0x00040000UL
#define GETB_FLAGS_WOL_ENABLED      0x00080000UL

typedef enum __get_init_type {
    GETB_INIT_COLD=0,
    GETB_INIT_RESET,
    GETB_INIT_WOL
} GETB_INIT_TYPE, *PGETB_INIT_TYPE;

#define GETB_MEMIO_RES_ID       0x14


#if __FreeBSD_version >= 400000

typedef struct __get_info {
    struct  arpcom              arpcom;
    int                         unit;
    struct resource*            mem_res;

    struct __get_info*          next;
    struct __get_info*          prev;

    struct resource*            int_res;
    void*                       int_hdl_tag;

    struct ifmedia              media;
    struct callout_handle       timer_handle;


    void*                       pool;
    int                         pool_size;

    CHIP_TYPE                   chip_id;


    PGETB_TD_INFO               apTDInfos[TX_QUEUE_NO];
    PGETB_RD_INFO               aRDInfo;


    U32                         adwHWMIBCounters[MAX_HW_MIB_COUNTER];

    int                         rx_buf_sz;
    int                         multicast_limit;


#ifdef CONFIG_PROC_FS
    PGETB_PROC_ENTRY            pProcDir;
#endif

    U32                         ticks;
    U32                         rx_bytes;

#if __FreeBSD_version >= 500000
    struct mtx                  get_mtx;
#else
    int                         get_splpre;
    int                         get_splcount;
#endif

    struct velocity_hw          hw;
    struct em_osdep             osdep;

} GETB_INFO, *PGETB_INFO;

#elif __FreeBSD_version < 400000

typedef struct __get_info {
    struct  arpcom              arpcom;
    struct  ifmedia             media;        /* media info */

    int                         unit;

    struct __get_info*          next;
    struct __get_info*          prev;

    caddr_t                     rawpool[TX_QUEUE_NO+1];
    void*                       pool[TX_QUEUE_NO+1];
    int                         pool_size;


    CHIP_TYPE                   chip_id;

    PGETB_TD_INFO               apTDInfos[TX_QUEUE_NO];

    PGETB_RD_INFO               aRDInfo;



    int                         rx_buf_sz;
    int                         multicast_limit;


#ifdef CONFIG_PROC_FS
    PGETB_PROC_ENTRY            pProcDir;
#endif

    U32                         ticks;
    U32                         rx_bytes;

    int                         get_splpre;
    int                         get_splcount;

    struct velocity_hw          hw;
    struct em_osdep             osdep;

} GETB_INFO, *PGETB_INFO;

#endif /*__FreeBSD_version*/

#if __FreeBSD_version >= 500000
#define GET_LOCK(_sc)           mtx_lock(&(_sc)->get_mtx)
#define GET_UNLOCK(_sc)         mtx_unlock(&(_sc)->get_mtx)
#else
#define GET_LOCK(_sc)           if ((_sc)->get_splcount==0) {     \
                                    (_sc)->get_splpre = splimp(); \
                                    (_sc)->get_splcount++;        \
                                } else {                          \
                                    (_sc)->get_splcount++;        \
                                }

#define GET_UNLOCK(_sc)         if ((_sc)->get_splcount>1) {      \
                                    (_sc)->get_splcount--;        \
                                } else {                          \
                                    (_sc)->get_splcount=0;        \
                                    splx((_sc)->get_splpre);      \
                                }
#endif

typedef struct __get_vendor_info {
    U16         wVendorId;
    U16         wDeviceId;
    U16         wSubVenId;
    U16         wSubDevId;
    PCHIP_INFO  pChip;
} GETB_VENDOR_INFO, *PGETB_VENDOR_INFO;



/*
 * PCI low memory base and low I/O base register, and
 * other PCI registers.
 */
#define GETB_PCI_VENDOR_ID       0x00
#define GETB_PCI_DEVICE_ID       0x02
#define GETB_PCI_COMMAND         0x04
#define GETB_PCI_STATUS          0x06
#define GETB_PCI_REVID           0x08
#define GETB_PCI_CLASSCODE       0x09
#define GETB_PCI_LATENCY_TIMER   0x0D
#define GETB_PCI_HEADER_TYPE     0x0E
#define GETB_PCI_LOIO            0x10
#define GETB_PCI_LOMEM           0x14
#define GETB_PCI_SUBVEND         0x2C
#define GETB_PCI_SUBDEV          0x2E
#define GETB_PCI_BIOSROM         0x30
#define GETB_PCI_INTLINE         0x3C
#define GETB_PCI_INTPIN          0x3D
#define GETB_PCI_MINGNT          0x3E
#define GETB_PCI_MINLAT          0x0F
#define GETB_PCI_RESETOPT        0x48
#define GETB_PCI_EEPROM_DATA     0x4C
#define GETB_PCI_MODE            0x50

#define PCI_VENDORID(x)     ((x) & 0xFFFF)
#define PCI_CHIPID(x)       (((x) >> 16) & 0xFFFF)

#endif /* __IF_GETB_H__ */

